/*global define, ep*/
/*jslint nomen:true*/
define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/externalcontent/ui/otherbuttons-test.html',
    'jquery',

    'de_epages/externalcontent/ui/otherbuttons',
    '$ready!'
], function (QUnit, fixture, $) {
    'use strict';

    QUnit.module('de_epages/externalcontent/ui/otherbuttons', {
        setup: function () {
            fixture.init();
            ep.config = {
                storeRoot: '..',
                canonicalUrl: 'http://canonicalurl.com',
                webUrl: 'http://weburl.com'
            };
        },
        teardown: fixture.destroy
    });

    QUnit.test('Test otherbuttons methods', 2, function () {
        $.each(['_create', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.externalcontentUiOtherbuttons.prototype[fn]),
                true,
                fn + ' is a method of otherbuttons'
            );
        });
    });

    QUnit.test('Test otherbuttons', function () {
        // Setup of test options.
        var letters = ['A', 'B'],
            services = {
                A: ['digg', 'delicious', 'stumbleupon'],
                B: ['digg']
            },

            responseString = '["?ViewAction=DereferredRedirect&URL=http%3A//digg.com/submit%3Furl%3Dhttp%3A//ep6unity2.vm-intern.epages.com/epages/qp-ep6.sf&nonce=YjM5NWY5YTc1ZmM5ZGM1NTJmZTI4ODJkMjQwZWM4MGFkYzUxODhkNDY5Zjg2MjE3MDFlMGEwOWY2MGVjZTMyOF8xNDg2NDY4OTUz","?ViewAction=DereferredRedirect&URL=http%3A//www.delicious.com/save%3Furl%3Dhttp%3A//ep6unity2.vm-intern.epages.com/epag…AzYThjNjUzMzk3NDY5ZWQwNjFjOTE2NzYxNzAyMGZiM2Q2MDI0ZjgyNzgyY18xNDg2NDY4OTUz","?ViewAction=DereferredRedirect&URL=http%3A//www.stumbleupon.com/submit%3Furl%3Dhttp%3A//ep6unity2.vm-intern.epages.com/…c3NTRlZTMwNjEzMGVjN2Y4NmM4OGM0YjJiZWQ5YzdkNDdjNDEyYTBiNDk4Ml8xNDg2NDY4OTUz"]',
            response = JSON.parse(responseString);

        /* START - fake ajax request */
        ep.ajax = function (o) {
            var result = $.Deferred();

            if (o) {
                result.resolve(response);
            } else {
                result.reject();
            }

            return result;
        };
        /* END - fake ajax request */

        // Create three *otherbuttons* gadgets.
        $.each(letters, function (index, value) {
            if (index === 1) {
                // Here we want to check whether the webUrl is used in case there is no canonicalUrl.
                ep.config.canonicalUrl = null;
            }

            $('div#' + value).externalcontentUiOtherbuttons({
                'services': services[value]
            });

            if (index === 1) {
                ep.config.canonicalUrl = 'http://canonicalurl.com';
            }
        });

        // Testing *_create*.
        $.each(letters, function (index, value) {
            var $gNode = $('div#' + value);

            QUnit.equal(
                $gNode.children().length,
                services[value].length,
                'Gadget #' + value + ' has the right number of bookmark buttons: $gNode.children().length'
            );

            $.each(services[value], function (_index, _value) {
                QUnit.equal(
                    (new RegExp(_value)).test($($gNode.children()[_index]).attr('href')),
                    true,
                    'Gadget #' + value + '\'s *href* attribute ends with the right url:' + ((index !== 1) ? '*ep.config.canonicalUrl*' : '*ep.config.webUrl* (Fallback to *webUrl* works!)')
                );
            });
        });

        //Testing *destroy*.
        $('div#qunit-fixture div').each(function () {
            if ($(this).data('uiExternalcontentUiOtherbuttons')) {
                $(this).externalcontentUiOtherbuttons('destroy');

                QUnit.equal(
                    $(this).children().length,
                    0,
                    'Destroy works. All *children* are gone.'
                );

                QUnit.equal(
                    $(this).attr('class'),
                    "",
                    'Destroy works. CSS classes added by the gadget were removed.'
                );
            }
        });
    });
});