/*globals define*/

define([
    'testsuite/qunit',
    'jquery',
    'de_epages/presentation/tray/selected-view',
    'de_epages/presentation/tray/tray-model',
    'de_epages/presentation/tray/tray-collection',
    '$dict!de_epages/presentation/dictionary',
    '$ready!'
], function (QUnit, $, ContentSelectedView, TrayModel, TrayCollection, dict) {
    'use strict';

    var fixture = $('#qunit-fixture'),
        view,
        collection;

    QUnit.module("de_epages/presentation/tray/selected-view", {
        setup: function () {
            collection = new TrayCollection();
            collection.add([
                {
                NameOrAlias: 'One',
                ObjectID: 1,
                type: 'products'
                },
                {
                NameOrAlias: 'Two',
                ObjectID: 2,
                type: 'users'
                },
                {
                NameOrAlias: 'Three',
                ObjectID: 3,
                type: 'users'
                },
                {
                NameOrAlias: 'Four',
                ObjectID: 4,
                type: 'products'
                },
                {
                NameOrAlias: 'Five',
                ObjectID: 5,
                type: 'users'
                }
            ]);

            view = new ContentSelectedView({
                collection: collection
            });

            view.render().$el.appendTo(fixture);
        },

        teardown: function () {
            fixture.empty();
        }
    });

    QUnit.test('Check content-selected-view\'s public methods', function () {
        $.each(['initialize', 'render'], function () {
            QUnit.ok(
                $.isFunction(view[this]),
                'Check if ' + this + ' is a method of the view'
            );
        });
    });

    QUnit.test('The DOM', function () {
        var nameOrAliases, types;

        QUnit.equal(
            fixture.find('div.ui-accordion > h3').length,
            2,
            'should have one h3 for each type'
        );

        types = ['Products', 'Users'];
        fixture.find('div.ui-accordion > h3').each(function (i, h3) {
            h3 = $(h3);
            QUnit.equal(
                h3.text().trim(),
                dict.translate(types[i]),
                'should have "' + dict.translate(types[i]) + '" as the content of the ' + (i + 1) + '. h3 for translation purposes'
            );
        });

        QUnit.equal(
            fixture.find('div.ui-accordion > div.ui-accordion-content').length,
            2,
            'should have one ui-accordion-content div for each type'
        );

        QUnit.equal(
            fixture.find('li').length,
            5,
            'should have one li for each TrayModel'
        );

        nameOrAliases = ['One', 'Four', 'Two', 'Three', 'Five'];
        fixture.find('div.ui-accordion li').each(function (i, li) {
            li = $(li);

            QUnit.equal(
                nameOrAliases[i],
                li.text().trim(),
                'should have "' + nameOrAliases[i] + '" as the content of the ' + i + 'th li'
            );
        });
    });

    QUnit.test('Removing items', function () {
        fixture.find('ul li i[data-id=1]').trigger('click');

        QUnit.equal(
            fixture.find('li').length,
            4,
            'should have 4 instead of 5 lis after the deletion of one item'
        );
    });
});